#/bin/bash -x

# This script builds GCC for r5900 (EE) for Playstation 2

cd "`dirname \"$0\"`"
SCRIPTDIR="`pwd`"
BASEDIR="`dirname \"$SCRIPTDIR\"`"
WORKDIR="$BASEDIR/build"
SRCPREFIX="$BASEDIR/src"

#TARGET=mips64r5900el-unknown-linux-uclibc
TARGET=mips64r5900el-unknown-linux-gnu
#TARGET=mips64r5900el-linux-gnu
#TARGET=mips64r5900el-scei-elf
#TARGET=mips64r5900el-unknown-elf
PREFIX=/usr/local/ps2linux
PROGRAMPREFIX=ee-
GCCBASE=4.2
GCCRELEASE=0
GCCVERSION="$GCCBASE.$GCCRELEASE"
PS2OVERLAY="$BASEDIR/ps2-overlay-gcc-$GCCBASE"
ULIBC=0

echo "WORKDIR: $WORKDIR"
echo "BASEDIR: $BASEDIR"
echo "SRCPREFIX: $SRCPREFIX"

mkdir -p "$WORKDIR" || exit -1
mkdir -p "$SRCPREFIX" || exit -1

check_program()
{
	PROGRAM="$1"
	which $PROGRAM >/dev/null
	if [ $? -ne 0 ]; then
		echo "The program $PROGRAM is not installed. Please install it."
		exit -4
	fi
}

load_source()
{
	check_program md5sum

	cd "$SRCPREFIX" || exit -5
	md5sum -c gcc-$GCCVERSION.md5
	if [ $? -ne 0 ]; then
		echo "Loading GCC..."
		check_program wget
		wget -P "$SRCPREFIX" ftp://ftp.gwdg.de/pub/misc/gcc/releases/gcc-$GCCVERSION/gcc-$GCCVERSION.tar.bz2 || exit -1
	fi
	md5sum -c binutils-2.16.1.md5
	if [ $? -ne 0 ]; then
		echo "Loading Binutils..."
		check_program wget
		wget -P "$SRCPREFIX" http://ftp.gnu.org/gnu/binutils/binutils-2.16.1.tar.bz2 || exit -1
	fi

	#cd "$BASEDIR"
	#check_program svn
	#svn co svn://gps2.aura-online.co.uk/gentoo-ps2/ps2-overlay || exit -1
	#cd "$WORDIR"
	#check_program wget
	#wget -P "$SRCPREFIX" ftp://ftp.gwdg.de/pub/misc/gcc/releases/gcc-$GCCVERSION/gcc-core-$GCCVERSION.tar.bz2 || exit -1
	#wget -P "$SRCPREFIX" ftp://ftp.gwdg.de/pub/misc/gcc/releases/gcc-$GCCVERSION/gcc-g++-$GCCVERSION.tar.bz2 || exit -1
	#wget -P "$SRCPREFIX" http://dev.gentoo.org/~vapier/dist/binutils-2.16.1-patches-1.4.tar.bz2 || exit -1
	#cvs -d :pserver:anonymous@anoncvs.gentoo.org:/var/cvsroot co gentoo/src/patchsets/binutils/2.16.1
	#wget -P "$SRCPREFIX" http://dev.gentoo.org/~vapier/dist/gcc-$GCCVERSION-patches-1.4.tar.bz2 || exit -1
	#cd $BASEDIR/gentoo-ps2-patches && wget -P "$SRCPREFIX" http://gps2.aura-online.co.uk/distfiles/binutils-2.16.1-patches-1.14.tar.bz2 || exit -1
	#cd $BASEDIR && cvs -d :pserver:anonymous@anoncvs.gentoo.org:/var/cvsroot co gentoo/src/patchsets/gcc/$GCCVERSION || exit -1
}

clean_up()
{
	cd "$WORKDIR"
	
	rm -rf binutils-2.16.1 || exit -1
	rm -rf gcc-$GCCVERSION || exit -2
	rm -rf patch || exit -2
	rm -rf binutils-patch || exit -2
	#rm -rf gcc-patch || exit -2
}

extract_binutils()
{
	cd "$WORKDIR"
	tar -xjf $SRCPREFIX/binutils-2.16.1.tar.bz2 || exit -3
	tar -xjf "$BASEDIR/gentoo-ps2-patches/binutils-2.16.1-patches-1.14.tar.bz2" || exit -89
	mv patch binutils-patch || exit -90
}

extract_gcc()
{
	cd "$WORKDIR"
	#tar -xjf $SRCPREFIX/gcc-$GCCVERSION-patches-1.4.tar.bz2 || exit -89
	#mv patch gcc-patch || exit -90
	#rm gcc-patch/90_all_mips-add-march-r10k.patch || exit -90
	#rm gcc-patch/91_all_mips-ip28_cache_barriers-v4.patch || exit -90
	#tar -xjf $SRCPREFIX/gcc-core-$GCCVERSION.tar.bz2 || exit -4
	#tar -xjf $SRCPREFIX/gcc-g++-$GCCVERSION.tar.bz2 || exit -4
	tar -xjf $SRCPREFIX/gcc-$GCCVERSION.tar.bz2 || exit -4
}

patch_binutils()
{
	cd "$WORKDIR/binutils-2.16.1" || exit -8
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/02_all_binutils-2.15.92.0.2-ltconfig-multilib.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/02_all_binutils-2.16.1-cross-multi-libsuffix.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/03_all_binutils-2.15.92.0.2-ppc64-pie.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/04_all_binutils-2.15.92.0.2-place-orphan.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/05_all_binutils-2.16-arm-updates.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/06_all_bintuils-head-mips-trim-page-alignment-bloat.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/10_all_generate-pie-scripts.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/11_all_sh-flags-off-by-one.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/12_all_sh-targets.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/30_all_ld-pr2290-search-order.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/31_all_ld-pr2218-weak-undefined-PIE_part0.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/31_all_ld-pr2218-weak-undefined-PIE_part1.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/31_all_ld-pr2218-weak-undefined-PIE_part2.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/31_all_ld-pr2218-weak-undefined-PIE_part3.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/40_all_118_arm_pass_all.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/40_all_binutils-2.16-branch-all-libcs.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/42_all_012_check_ldrunpath_length.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/53_all_binutils-avr-dollar-sign-in-symbol.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/54_all_binutils-avr-new-devices.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/55_all_binutils-2.16.94-ld-lazy-flag.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/63_all_binutils-2.16.1-pt-pax-flags-20051005.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/65_all_binutils-2.14-amd64-32bit-path.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/66_all_binutils-2.16-warn-textrel.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/73_all_ld-speedup-callahan.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/74_all_bfd-speedup-stabs-tweak.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/76_all_use-new-ld-dtags.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/81_all_binutils-2.16-readelf-ptr.patch" || exit -20
	#patch -p0 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/91_all_libiberty-pic.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/95_all_binutils-2.16.1-freebsd-ldhints.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/96_all_binutils-2.16.1-freebsd-amd64.patch" || exit -20
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/binutils/2.16.1/97_all_binutils-2.16.1-playstation2.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/02_all_binutils-2.15.92.0.2-ltconfig-multilib.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/02_all_binutils-2.16.1-cross-multi-libsuffix.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/03_all_binutils-2.15.92.0.2-ppc64-pie.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/04_all_binutils-2.15.92.0.2-place-orphan.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/05_all_binutils-2.16-arm-updates.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/06_all_bintuils-head-mips-trim-page-alignment-bloat.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/10_all_generate-pie-scripts.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/11_all_sh-flags-off-by-one.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/12_all_sh-targets.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/30_all_ld-pr2290-search-order.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/31_all_ld-pr2218-weak-undefined-PIE_part0.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/31_all_ld-pr2218-weak-undefined-PIE_part1.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/31_all_ld-pr2218-weak-undefined-PIE_part2.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/31_all_ld-pr2218-weak-undefined-PIE_part3.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/40_all_118_arm_pass_all.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/40_all_binutils-2.16-branch-all-libcs.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/41_all_006_better_file_error.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/42_all_012_check_ldrunpath_length.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/53_all_binutils-avr-dollar-sign-in-symbol.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/54_all_binutils-avr-new-devices.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/55_all_binutils-2.16.94-ld-lazy-flag.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/63_all_binutils-2.16.1-pt-pax-flags-20051005.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/65_all_binutils-2.14-amd64-32bit-path.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/66_all_binutils-2.16-warn-textrel.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/73_all_ld-speedup-callahan.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/74_all_bfd-speedup-stabs-tweak.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/76_all_use-new-ld-dtags.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/81_all_binutils-2.16-readelf-ptr.patch" || exit -20
	patch -p0 <"$WORKDIR/binutils-patch/91_all_libiberty-pic.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/95_all_binutils-2.16.1-freebsd-ldhints.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/96_all_binutils-2.16.1-freebsd-amd64.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/97_all_binutils-2.16.1-r5900.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/98_all_binutils-2.16.1-dvp.patch" || exit -20
	patch -p1 <"$WORKDIR/binutils-patch/99_all_binutils-2.16.1-irx.patch" || exit -20
}

patch_gcc()
{
	# Patching
	cd "$WORKDIR/gcc-$GCCVERSION" || exit -5
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/00_all_gcc-4.1-alpha-mieee-default.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/00_all_gcc-trampolinewarn.patch" || exit -19
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/01_all_gcc4-ice-hack.patch" || exit -19
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/02_all_gcc4-ppc64-m32-m64-multilib-only.patch" || exit -19
	# XXX: Can't patch libjava/configure.ac
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/03_all_gcc4-java-nomulti.patch" || exit -19
	# XXX: Can't patch libjava/configure.host
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/06_all_gcc4-slow-pthread-self.patch" || exit -19
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/08_all_gcc-4.1-cross-compile.patch" || exit -19
	# XXX: Can't patch libjava/libltdl/acinclude.m4
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/10_all_gcc4-libltdl-multilib.patch" || exit -19
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/11_all_gcc-netbsd-symbolic.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/20_all_cris-dont-force-limits-header.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/34_all_arm-bigendian.patch" || exit -19
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/40_all_gcc42-PR30052.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/41_all_gcc42-PR32389.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/42_all_gcc42-PR30252.patch" || exit -19
	patch -p2 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/43_all_gcc42-PR32500.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/51_all_gcc-3.4-libiberty-pic.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/53_all_gcc4-superh-default-multilib.patch" || exit -19
	# XXX: Can't patch gcc/libstdc++-v3/src/Makefile.am
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/54_all_300-libstdc++-pic.patch" || exit -19
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/61_all_gcc4-ia64-noteGNUstack.patch" || exit -19
	# XXX: Can't patch libffi/src/alpha/osf.S
	patch -p0 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/62_all_gcc4-noteGNUstack.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/74_all_sh-pr24836.patch" || exit -19
	patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/85_all_gcc-ca-translation-typo.patch" || exit -19
	# XXX: Can't patch gcc-$GCCVERSION/gcc/config/mips/10000.md
	#patch -p2 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/90_all_mips-add-march-r10k.patch" || exit -19
	#patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/gentoo/91_all_mips-ip28_cache_barriers-v4.patch" || exit -19
	
	
	if [ "$ULIBC" != "0" ]; then
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_100-uclibc-conf.patch" || exit -19
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_103-uclibc-conf-noupstream.patch" || exit -19
		# XXX: Can't patch gcc/libstdc++-v3/acinclude.m4
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_200-uclibc-locale.patch" || exit -19
		# XXX: Can't patch gcc/libstdc++-v3/config/locale/uclibc/c++locale_internal.h
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_203-uclibc-locale-no__x.patch" || exit -19
		# XXX: Can't patch gcc/libstdc++-v3/config/locale/uclibc/monetary_members.cc
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_204-uclibc-locale-wchar_fix.patch" || exit -19
		# XXX: Can't patch gcc/libstdc++-v3/config/locale/uclibc/c_locale.cc
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_205-uclibc-locale-update.patch" || exit -19
		# XXX: Can't patch gcc-4.0.0/boehm-gc/include/gc.h
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_301-missing-execinfo_h.patch" || exit -19
		# XXX: Can't patch gcc-4.0.0/libstdc++-v3/include/c_std/std_cstdio.h
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_302-c99-snprintf.patch" || exit -19
		patch -p2 <"$BASEDIR/gentoo/src/patchsets/gcc//uclibc/90_all_303-c99-complex-ugly-hack.patch" || exit -19
		# XXX: Can't patch gcc-4.1.0/libstdc++-v3/include/ext/rope
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_304-index_macro.patch" || exit -19
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_305-libmudflap-susv3-legacy.patch" || exit -19
		# XXX: Can't patch gcc-4.2/libstdc++-v3/config/locale/uclibc/messages_members.h
		patch -p2 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_306-libstdc++-namespace.patch" || exit -19
		patch -p1 <"$BASEDIR/gentoo/src/patchsets/gcc/$GCCVERSION/uclibc/90_all_402-libbackend_dep_gcov-iov.h.patch" || exit -19
	fi
	
	patch -p1 <"$PS2OVERLAY/sys-devel/gcc/files/$GCCVERSION/gcc-4.2.0-r5900.patch" || exit -5
	patch -p1 <"$PS2OVERLAY/sys-devel/gcc/files/$GCCVERSION/gcc-4.2.0-iop.patch" || exit -6
}

configure_binutils()
{
	cd "$WORKDIR/binutils-2.16.1" || exit -8
	./configure --target=$TARGET --prefix=$PREFIX --program-prefix=$PROGRAMPREFIX \
		--enable-shared --disable-nls --enable-64-bit-bfd --disable-werror || exit -9
}

build_binutils()
{
	cd "$WORKDIR/binutils-2.16.1" || exit -8
	make || exit -10
	sudo make install || exit -11
}

configure_gcc()
{
	cd "$WORKDIR/gcc-$GCCVERSION" || exit -13
	./configure --target=$TARGET --prefix=$PREFIX --program-prefix=$PROGRAMPREFIX \
	--enable-languages=c --enable-shared --disable-nls --enable-64-bit-bfd \
	--disable-werror --with-system-zlib --enable-secureplt \
	--disable-libunwind-exceptions --disable-checking --enable-__cxa_atexit \
	--disable-threads || exit -14
	# --without-headers --with-newlib ???
}

build_gcc()
{
	cd "$WORKDIR/gcc-$GCCVERSION" || exit -13
	make || exit -11
	sudo make install || exit -11
}


# Check installed programs
check_program which
check_program tar
check_program gcc
check_program patch
check_program sudo

# Load source from internet
load_source

# Clean up last build
clean_up

# Build binutils
extract_binutils
patch_binutils
configure_binutils
build_binutils

# Build GCC
extract_gcc
patch_gcc
configure_gcc
build_gcc


